// niz2.h  Klasa nizova apstraktnih podataka.

#ifndef _niz2_h_
#define _niz2_h_

#include "podatak.h"
#include "niz2gr.h"

class Niz: public Podatak {   // Polja:
  Podatak** niz;              // - niz pokazivaca na podatke,
  int kap;                    // - kapacitet niza.
                                             // Pomocne metode:
  void kopiraj(const Niz& n);                // - kopiranje u niz,
  void premesti(Niz& n)                      // - premestanje u niz,
    { niz = n.niz; kap = n.kap; n.niz = nullptr; }
  void brisi() { ~*this; delete [] niz; }    // - oslobadjanje memorije,
  void pisi(ostream& it) const override;     // - pisanje niza.
public:                                      // Konstruktori:
  explicit Niz(int kk=10);                   // - praznog niza,
  Niz(const Niz& nn) { kopiraj(nn); }        // - kopirajuci,
  Niz(Niz&& nn) { premesti(nn); }            // - premestajuci.
  ~Niz() { brisi(); }                        // Destruktor.
  Niz& operator=(const Niz& n) {             // Dodela vrednosti.
    if (this != &n) { brisi(); kopiraj(n); } // - kopiranjem,
    return *this;
  }
  Niz& operator=(Niz&& n) {                  // - premestanjem.
    if (this != &n) { brisi(); premesti(n); }
    return *this;
  }
  Niz* kopija() const override               // Kopija niza.
    { return new Niz(*this); }
  int kapac() const { return kap; }          // Kapacitet niza.
                                             // Pristup komponenti:
  Podatak& operator[](int ind) {             // - promenljivog niza,
    if (ind < 0 || ind >= kap) throw G_indeks(ind, kap);
    if (niz[ind] == nullptr)   throw G_prazno();
    return *niz[ind];
  }
  const Podatak& operator[](int ind) const   // - nepromenljivog niza.
    { return const_cast<Niz&>(*this)[ind]; }
  Niz& operator+=(const Podatak& p);         // Umetanje podatka.
  Niz& operator-=(int ind) {                 // Izbacivanje podatka.
    if (ind < 0 || ind >= kap) throw G_indeks(ind, kap);
    delete niz[ind]; niz[ind] = nullptr;
    return *this;
  }
  Niz& operator~();                          // Praznjenje niza.
};

#endif
