// vektor2.h - Klasa vektora realnih brojeva.

#ifndef _vektor2_h_
#define _vektor2_h_

class Vekt {                            // Polja:
  int min, max;                         // - granice indeksa,
  int duz;                              // - broj komponenata,
  double* niz;                          // - niz komponenata.
                                        // Pomocne metode:
  void pravi(int poc, int kra);         // - formiranje vektora,
  void kopiraj(const Vekt& v);          // - kopiranje u vektor,
  void premesti(Vekt& v) {              // - premestanje u vektor,
    min = v.min; max = v.max; duz = v.duz;
    niz = v.niz; v.niz = nullptr;
  }
  void brisi()                          // - oslobadjanje memorije.
    { delete[] niz; niz = nullptr; }
public:                                 // Kodovi gresaka:
  enum Greska { OPSEG,                  // - neispravan opseg indeksa,
                PRAZAN,                 // - vektor je prazan,
                INDEKS,                 // - indeks je izvan opsega,
                DUZINA };               // - neusaglasene duzine vektora.
                                             // Konstruktori:
  Vekt() { niz = nullptr; }                  // - prazan niz,
  explicit Vekt(int kra) { pravi(1, kra); }  // - podraz. pocetni indeks,
  Vekt(int poc, int kra) { pravi(poc,kra); } // - sa zadatim indeksima,
  Vekt(const Vekt& v) { kopiraj(v); }        // - kopiranjem,
  Vekt(Vekt&& v) { premesti(v); }            // - premestanjem,
  ~Vekt() { brisi(); }                       // Destruktor.
  Vekt& operator=(const Vekt& v) {           // Dodela vrednosti:
    if (this != &v) { brisi(); kopiraj(v); } // - kopiranjem,
    return *this;
  }
  Vekt& operator=(Vekt&& v) {                // - premestanjem.
    if (this != &v) { brisi(); kopiraj(v); }
    return *this;
  }                                          // Pristup komponenti:
  double& operator[](int ind) {              // - promenljivog vektora,
    if (niz == nullptr)     throw PRAZAN;
    if (ind<min || ind>max) throw INDEKS;
    return niz[ind-min];
  }
 
  const double& operator[](int ind) const    // - nepromenljivog vektora.
    { return const_cast<Vekt&>(*this)[ind]; }
  friend double operator*                    // Skalarni proizvod.
    (const Vekt& v, const Vekt& w);
  int minInd() const { return min; }         // Najmanji indeks.
  int maxInd() const { return max; }         // Najveci indeks.
};

#endif

